# Copyright 2021 Huawei, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib._i18n import _
from neutron_lib import exceptions


class LocalIPNotFound(exceptions.NotFound):
    message = _("Local IP %(id)s could not be found.")


class LocalIPInUse(exceptions.InUse):
    message = _("Local IP %(id)s is still associated with one or more ports.")


class LocalIPNoIP(exceptions.InvalidInput):
    message = _("Specified Port %(port_id)s has no fixed IPs configured.")


class LocalIPRequestedIPNotFound(exceptions.InvalidInput):
    message = _("Specified Port %(port_id)s does not have "
                "requested IP address: %(ip)s.")


class LocalIPNoRequestedIP(exceptions.InvalidInput):
    message = _("Specified Port %(port_id)s has several IPs, "
                "should specify exact IP address to use for Local IP.")


class LocalIPPortOrNetworkRequired(exceptions.InvalidInput):
    message = _("Either Port ID or Network ID must be specified "
                "for Local IP create.")


class LocalIPAssociationNotFound(exceptions.NotFound):
    message = _("Local IP %(local_ip_id)s association with port %(port_id)s "
                "could not be found.")



# Copyright 2023 EasyStack Limited. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import allowedaddresspairs
from neutron_lib.api.definitions import port


ALIAS = 'allowed-address-pairs-atomic'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Atomically add/remove allowed address pairs'
DESCRIPTION = ('Edit allowed address pairs of a port on server side by '
               'atomically adding/removing allowed address pairs')
UPDATED_TIMESTAMP = '2023-04-17T00:00:00+00:00'
RESOURCE_ATTRIBUTE_MAP = {
    port.COLLECTION_NAME: {}
}
SUB_RESOURCE_ATTRIBUTE_MAP = None
ACTION_MAP = {
    port.RESOURCE_NAME: {
        'add_allowed_address_pairs': 'PUT',
        'remove_allowed_address_pairs': 'PUT',
    }
}
REQUIRED_EXTENSIONS = [allowedaddresspairs.ALIAS]
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# Copyright (c) 2021 Ericsson Software Technology
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters
from neutron_lib.api.definitions import qos
from neutron_lib.api.definitions import qos_pps_minimum_rule
from neutron_lib import constants
from neutron_lib.db import constants as db_const
from neutron_lib.services.qos import constants as qos_constants


MIN_PACKET_RATE_RULES_ALIAS = 'alias_minimum_packet_rate_rules'

ALIAS = 'qos-pps-minimum-rule-alias'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'QoS minimum packet rate rule alias'
API_PREFIX = '/' + qos.ALIAS
DESCRIPTION = ('API to enable GET, PUT and DELETE operations on QoS minimum '
               'packet rate rule without specifying policy ID')
UPDATED_TIMESTAMP = '2021-10-22T10:00:00-00:00'
RESOURCE_ATTRIBUTE_MAP = {
    MIN_PACKET_RATE_RULES_ALIAS: {
        **qos._QOS_RULE_COMMON_FIELDS,
        qos_constants.MIN_KPPS: {
                'allow_post': False,
                'allow_put': True,
                'convert_to': converters.convert_to_int,
                'is_visible': True,
                'is_filter': True,
                'is_sort_key': True,
                'validate': {
                    'type:range': [0, db_const.DB_INTEGER_MAX_VALUE]}
        },
        qos_constants.DIRECTION: {
            'allow_post': False,
            'allow_put': True,
            'is_visible': True,
            'default': constants.EGRESS_DIRECTION,
            'is_filter': True,
            'is_sort_key': True,
            'validate': {
                'type:values': constants.VALID_DIRECTIONS_AND_ANY,
            }
        }
    }
}
SUB_RESOURCE_ATTRIBUTE_MAP = {}
ACTION_MAP = {}
REQUIRED_EXTENSIONS = [qos.ALIAS, qos_pps_minimum_rule.ALIAS]
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# Copyright (c) 2013 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron_lib.api.definitions import agent

ALIAS = 'agent_sort_key'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Agents Sort Key'
API_PREFIX = ''
DESCRIPTION = 'Enabling the keyword sort_key for sorting in agents.'
UPDATED_TIMESTAMP = '2013-02-03T10:00:00-00:00'
is_sort_key = {'is_sort_key': True}
agents_collection = agent.RESOURCE_ATTRIBUTE_MAP[agent.COLLECTION_NAME]
RESOURCE_ATTRIBUTE_MAP = {
    agent.COLLECTION_NAME: {
        k: {**v, **is_sort_key}
        for k, v in agents_collection.items()
        if k != "configurations"
    }
}
SUB_RESOURCE_ATTRIBUTE_MAP = {}
ACTION_MAP = {}
REQUIRED_EXTENSIONS = []
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import multiprocessing
from unittest import mock

from neutron_lib.tests import _base as base
from neutron_lib.utils import host


class TestCpuCount(base.BaseTestCase):

    @mock.patch.object(multiprocessing, 'cpu_count',
                       return_value=7)
    def test_cpu_count(self, mock_cpu_count):
        self.assertEqual(7, host.cpu_count())
        mock_cpu_count.assert_called_once_with()

    @mock.patch.object(multiprocessing, 'cpu_count',
                       side_effect=NotImplementedError())
    def test_cpu_count_not_implemented(self, mock_cpu_count):
        self.assertEqual(1, host.cpu_count())
        mock_cpu_count.assert_called_once_with()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron_lib.api import converters
from neutron_lib.api.definitions import network

ALIAS = 'network_ha'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Network HA creation flag'
API_PREFIX = ''
DESCRIPTION = 'Network high availability creation flag.'
UPDATED_TIMESTAMP = '2023-04-27T10:00:00-00:00'
RESOURCE_NAME = network.RESOURCE_NAME
COLLECTION_NAME = network.COLLECTION_NAME
HA = 'ha'
RESOURCE_ATTRIBUTE_MAP = {
    COLLECTION_NAME: {
        HA: {
            'allow_post': True,
            'allow_put': False,
            'is_visible': False,
            'default': False,
            'convert_to': converters.convert_to_boolean,
        },
    }
}
SUB_RESOURCE_ATTRIBUTE_MAP = {}
ACTION_MAP = {}
REQUIRED_EXTENSIONS = []
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

import neutron_lib.exceptions.l3_ext_ha_mode as lehm
from neutron_lib.tests.unit.exceptions import test_exceptions


class TestHANetworkConcurrentDeletion(test_exceptions.TestExceptionsBase):

    # NOTE(haleyb) We only test HANetworkConcurrentDeletion because it is
    # the only one with an __init__() section.
    def test_ha_network_concurrent_deletion(self):
        self._check_nexc(
            lehm.HANetworkConcurrentDeletion,
            _('Network for project project_id concurrently deleted.'),
            project_id='project_id')

    @mock.patch.object(lehm, 'LOG')
    def test_ha_network_concurrent_deletion_tenant_id(self, mock_log):
        self._check_nexc(
            lehm.HANetworkConcurrentDeletion,
            _('Network for project tenant_id concurrently deleted.'),
            tenant_id='tenant_id')
        mock_log.warning.assert_called_once_with(
            'Keyword tenant_id has been deprecated, use project_id instead')

    @mock.patch.object(lehm, 'LOG')
    def test_ha_network_concurrent_deletion_both(self, mock_log):
        self._check_nexc(
            lehm.HANetworkConcurrentDeletion,
            _('Network for project project_id concurrently deleted.'),
            project_id='project_id', tenant_id='tenant_id')
        mock_log.warning.assert_called_once_with(
            'Keyword tenant_id has been deprecated, use project_id instead')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from neutron_lib.objects import utils as obj_utils
from neutron_lib.tests import _base as base


class TestUtils(base.BaseTestCase):

    def test_get_objects_with_filters_not_in(self):

        class FakeColumn:
            def __init__(self, column):
                self.column = column

            def in_(self, value):
                self.value = value
                return self

            def __invert__(self):
                return list(set(self.column) - set(self.value))

        filter_obj = obj_utils.NotIn([1, 2, 3])
        fake_column = FakeColumn([1, 2, 4, 5])
        self.assertEqual([4, 5],
                         sorted(filter_obj.filter(fake_column)))

        fake_column = FakeColumn([1, 2])
        self.assertEqual([], filter_obj.filter(fake_column))

        fake_column = FakeColumn([4, 5])
        self.assertEqual([4, 5],
                         sorted(filter_obj.filter(fake_column)))

    def test_get_objects_with_filters_not_equal(self):

        class FakeColumn:
            def __init__(self, column):
                self.column = column

            def __ne__(self, value):
                return [item for item in self.column if item != value]

        filter_obj = obj_utils.NotEqual(1)
        fake_column = FakeColumn([1, 2, 4, 5])
        self.assertEqual([2, 4, 5],
                         sorted(filter_obj.filter(fake_column)))

        fake_column = FakeColumn([1])
        self.assertEqual([], filter_obj.filter(fake_column))

        fake_column = FakeColumn([4, 5])
        self.assertEqual([4, 5],
                         sorted(filter_obj.filter(fake_column)))

    def test_get_updatable_fields(self):
        mock_class = mock.Mock()
        mock_class.fields_no_update = [0, 2, 6]

        mock_fields = mock.Mock()
        mock_fields.copy.return_value = {k: k for k in range(7)}

        updatable = obj_utils.get_updatable_fields(mock_class, mock_fields)
        self.assertEqual([1, 3, 4, 5],
                         sorted(list(updatable.keys())))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Base classes for unit tests needing DB backend.
Only sqlite is supported in neutron-lib.
"""

from neutron_lib import fixture
from neutron_lib.tests import _base as base


class SqlTestCase(base.BaseTestCase):

    def setUp(self):
        super().setUp()
        self.useFixture(fixture.SqlFixture())



# Copyright 2023 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutron_lib._i18n import _
from neutron_lib import exceptions


class UnableToAddExtraGateways(exceptions.NeutronException):
    message = _("Unable to add extra gateways to a router %(router_id)s")


class UnableToRemoveGateways(exceptions.NeutronException):
    message = _("Unable to remove extra gateways from a router %(router_id)s")


class UnableToMatchGateways(exceptions.NeutronException):
    message = _("Unable to match a requested gateway port to"
                " existing gateway ports for router %(router_id)s")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import bgpvpn
from neutron_lib.api import validators
from neutron_lib.tests.unit.api.definitions import base


class BgpvpnDefinitionTestCase(base.DefinitionBaseTestCase):
    extension_module = bgpvpn
    extension_resources = (bgpvpn.COLLECTION_NAME,)
    extension_attributes = ('type', 'route_targets', 'import_targets',
                            'export_targets', 'route_distinguishers',
                            'networks', 'routers', 'router_id', 'network_id')
    extension_subresources = ('network_associations', 'router_associations')

    def _data_for_invalid_rtdt(self):
        values = [[':1'],
                  ['1:'],
                  ['42'],
                  ['65536:123456'],
                  ['123.456.789.123:65535'],
                  ['4294967296:65535'],
                  ['1.1.1.1:655351'],
                  ['4294967295:65536'],
                  [''],
                  ]
        yield from values

    def _data_for_valid_rtdt(self):
        values = [['1:1'],
                  ['1:4294967295'],
                  ['65535:0'],
                  ['65535:4294967295'],
                  ['1.1.1.1:1'],
                  ['1.1.1.1:65535'],
                  ['4294967295:0'],
                  ['65536:65535'],
                  ['4294967295:65535'],
                  ]
        yield from values

    def test_valid_rtrd(self):
        for rtrd in self._data_for_valid_rtdt():
            msg = validators.validate_list_of_regex_or_none(
                rtrd,
                bgpvpn.RTRD_REGEX)
            self.assertIsNone(msg)

    def test_invalid_rtrd(self):
        for rtrd in self._data_for_invalid_rtdt():
            msg = validators.validate_list_of_regex_or_none(
                rtrd,
                bgpvpn.RTRD_REGEX)
            self.assertIsNotNone(msg)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslotest import base

from neutron_lib.db import resource_extend
from neutron_lib import fixture


@resource_extend.has_resource_extenders
class _DBExtender:

    @resource_extend.extends('ExtendedA')
    def _extend_a(self, resp, db_obj):
        pass

    @resource_extend.extends('ExtendedB')
    def _extend_b(self, resp, db_obj):
        pass


class TestResourceExtendClass(base.BaseTestCase):

    def test_extends(self):
        self.assertIsNotNone(resource_extend.get_funcs('ExtendedA'))
        self.assertIsNotNone(resource_extend.get_funcs('ExtendedB'))


class TestResourceExtend(base.BaseTestCase):

    def setUp(self):
        super().setUp()
        self.useFixture(fixture.DBResourceExtendFixture())

    def test_register_funcs(self):
        resources = ['A', 'B', 'C']
        for r in resources:
            resource_extend.register_funcs(r, (lambda x: x,))

        for r in resources:
            self.assertIsNotNone(resource_extend.get_funcs(r))

    def test_apply_funcs(self):
        resources = ['A', 'B', 'C']
        callbacks = []

        def _cb(resp, db_obj):
            callbacks.append(resp)

        for r in resources:
            resource_extend.register_funcs(r, (_cb,))

        for r in resources:
            resource_extend.apply_funcs(r, None, None)

        self.assertEqual(3, len(callbacks))



# Copyright 2018 Ericsson
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock
import uuid

from neutron_lib.placement import utils as place_utils
from neutron_lib.tests import _base as base


class TestPlacementUtils(base.BaseTestCase):

    def setUp(self):
        super().setUp()

        self._uuid_ns = uuid.UUID('94fedd4d-1ce0-4bb3-9c9a-c9c0f56de154')

    def test_physnet_trait(self):
        self.assertEqual(
            'CUSTOM_PHYSNET_SOME_PHYSNET',
            place_utils.physnet_trait('some-physnet'))

    def test_vnic_type_trait(self):
        self.assertEqual(
            'CUSTOM_VNIC_TYPE_SOMEVNICTYPE',
            place_utils.vnic_type_trait('somevnictype'))

    def test_six_uuid5_literal(self):
        try:
            # assertNotRaises
            place_utils.six_uuid5(
                namespace=self._uuid_ns,
                name='may or may not be a unicode string' +
                     ' depending on Python version')
        except Exception:
            self.fail('could not generate uuid')

    def test_six_uuid5_unicode(self):
        try:
            # assertNotRaises
            place_utils.six_uuid5(
                namespace=self._uuid_ns,
                name='unicode string')
        except Exception:
            self.fail('could not generate uuid')

    def test_agent_resource_provider_uuid(self):
        try:
            # assertNotRaises
            place_utils.agent_resource_provider_uuid(
                namespace=self._uuid_ns,
                host='some host')
        except Exception:
            self.fail('could not generate agent resource provider uuid')

    def test_device_resource_provider_uuid(self):
        try:
            # assertNotRaises
            place_utils.device_resource_provider_uuid(
                namespace=self._uuid_ns,
                host='some host',
                device='some device')
        except Exception:
            self.fail('could not generate device resource provider uuid')

    def test_resource_request_group_uuid(self):
        try:
            # assertNotRaises
            place_utils.resource_request_group_uuid(
                namespace=self._uuid_ns,
                qos_rules=[
                    mock.MagicMock(id='fake_id_0'),
                    mock.MagicMock(id='fake_id_1')
                ])
        except Exception:
            self.fail('could not generate resource request group uuid')

    def test_agent_resource_provider_uuid_stable(self):
        uuid_a = place_utils.agent_resource_provider_uuid(
            namespace=self._uuid_ns,
            host='somehost')
        uuid_b = place_utils.agent_resource_provider_uuid(
            namespace=self._uuid_ns,
            host='somehost')
        self.assertEqual(uuid_a, uuid_b)

    def test_device_resource_provider_uuid_stable(self):
        uuid_a = place_utils.device_resource_provider_uuid(
            namespace=self._uuid_ns,
            host='somehost',
            device='some-device')
        uuid_b = place_utils.device_resource_provider_uuid(
            namespace=self._uuid_ns,
            host='somehost',
            device='some-device')
        self.assertEqual(uuid_a, uuid_b)

    def test_resource_request_group_uuid_stable(self):
        uuid_a = place_utils.resource_request_group_uuid(
            namespace=self._uuid_ns,
            qos_rules=[
                mock.MagicMock(id='fake_id_0'),
                mock.MagicMock(id='fake_id_1')
            ]
        )
        uuid_b = place_utils.resource_request_group_uuid(
            namespace=self._uuid_ns,
            qos_rules=[
                mock.MagicMock(id='fake_id_0'),
                mock.MagicMock(id='fake_id_1')
            ]
        )
        self.assertEqual(uuid_a, uuid_b)

    def test_parse_rp_bandwidths(self):
        self.assertEqual(
            {},
            place_utils.parse_rp_bandwidths([]),
        )

        self.assertEqual(
            {'eth0': {'egress': None, 'ingress': None}},
            place_utils.parse_rp_bandwidths(['eth0']),
        )

        self.assertEqual(
            {'eth0': {'egress': None, 'ingress': None}},
            place_utils.parse_rp_bandwidths(['eth0::']),
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_bandwidths,
            ['eth0::', 'eth0::'],
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_bandwidths,
            ['eth0:not a number:not a number'],
        )

        self.assertEqual(
            {'eth0': {'egress': 1, 'ingress': None}},
            place_utils.parse_rp_bandwidths(['eth0:1:']),
        )

        self.assertEqual(
            {'eth0': {'egress': None, 'ingress': 1}},
            place_utils.parse_rp_bandwidths(['eth0::1']),
        )

        self.assertEqual(
            {'eth0': {'egress': 1, 'ingress': 1}},
            place_utils.parse_rp_bandwidths(['eth0:1:1']),
        )

        self.assertEqual(
            {'eth0': {'egress': 1, 'ingress': 1},
             'eth1': {'egress': 10, 'ingress': 10}},
            place_utils.parse_rp_bandwidths(['eth0:1:1', 'eth1:10:10']),
        )

    def test_parse_rp_pp_with_direction(self):
        self.assertEqual(
            {},
            place_utils.parse_rp_pp_with_direction([], 'default_host'),
        )

        self.assertEqual(
            {'host0': {'egress': None, 'ingress': None}},
            place_utils.parse_rp_pp_with_direction(['host0'],
                                                   'default_host'),
        )

        self.assertEqual(
            {'host0': {'egress': None, 'ingress': None}},
            place_utils.parse_rp_pp_with_direction(['host0::'],
                                                   'default_host'),
        )

        self.assertEqual(
            {'default_host': {'egress': None, 'ingress': None}},
            place_utils.parse_rp_pp_with_direction(['::'],
                                                   'default_host'),
        )

        self.assertEqual(
            {'host0': {'egress': 1, 'ingress': None}},
            place_utils.parse_rp_pp_with_direction(['host0:1:'],
                                                   'default_host'),
        )

        self.assertEqual(
            {'host0': {'egress': None, 'ingress': 1}},
            place_utils.parse_rp_pp_with_direction(['host0::1'],
                                                   'default_host'),
        )

        self.assertEqual(
            {'host0': {'egress': 1, 'ingress': 1}},
            place_utils.parse_rp_pp_with_direction(['host0:1:1'],
                                                   'default_host'),
        )

        self.assertEqual(
            {'default_host': {'egress': 0, 'ingress': 0}},
            place_utils.parse_rp_pp_with_direction([':0:0'],
                                                   'default_host'),
        )

        self.assertEqual(
            {'host0': {'egress': 1, 'ingress': 1},
             'host1': {'egress': 10, 'ingress': 10}},
            place_utils.parse_rp_pp_with_direction(
                ['host0:1:1', 'host1:10:10'],
                'default_host'),
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_pp_with_direction,
            ['default_host::', '::'], 'default_host',
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_pp_with_direction,
            ['host0::', 'host0::'], 'default_host',
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_pp_with_direction,
            ['host0:not a number:not a number'], 'default_host',
        )

    def test_parse_rp_pp_without_direction(self):
        self.assertEqual(
            {},
            place_utils.parse_rp_pp_without_direction([], 'default_host'),
        )

        self.assertEqual(
            {'host0': {'any': None}},
            place_utils.parse_rp_pp_without_direction(['host0'],
                                                      'default_host'),
        )

        self.assertEqual(
            {'host0': {'any': None}},
            place_utils.parse_rp_pp_without_direction(['host0:'],
                                                      'default_host'),
        )

        self.assertEqual(
            {'host0': {'any': 1}},
            place_utils.parse_rp_pp_without_direction(['host0:1'],
                                                      'default_host'),
        )

        self.assertEqual(
            {'default_host': {'any': None}},
            place_utils.parse_rp_pp_without_direction([':'],
                                                      'default_host'),
        )

        self.assertEqual(
            {'default_host': {'any': 0}},
            place_utils.parse_rp_pp_without_direction([':0'],
                                                      'default_host'),
        )

        self.assertEqual(
            {'host0': {'any': 1},
             'host1': {'any': 10}},
            place_utils.parse_rp_pp_without_direction(
                ['host0:1', 'host1:10'],
                'default_host'),
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_pp_without_direction,
            ['default_host:', ':'], 'default_host',
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_pp_without_direction,
            ['host0:', 'host0:'], 'default_host',
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_pp_without_direction,
            ['host0:not a number'], 'default_host',
        )

    def test_parse_rp_options(self):
        self.assertEqual(
            {},
            place_utils._parse_rp_options([], tuple()),
        )

        self.assertEqual(
            {'outer_key0': {'inner_key0': None}},
            place_utils._parse_rp_options(['outer_key0'], ('inner_key0',)),
        )
        self.assertEqual(
            {'outer_key0': {'inner_key0': None},
             'outer_key1': {'inner_key0': None},
             'outer_key2': {'inner_key0': None},
             'outer_key3': {'inner_key0': None}},
            place_utils._parse_rp_options(
                ['outer_key0', 'outer_key1', 'outer_key2', 'outer_key3'],
                ('inner_key0',)),
        )

        self.assertEqual(
            {'outer_key0': {
                'inner_key0': None,
                'inner_key1': None,
                'inner_key2': None,
                'inner_key3': None}},
            place_utils._parse_rp_options(
                ['outer_key0'],
                ('inner_key0', 'inner_key1', 'inner_key2', 'inner_key3')),
        )

        self.assertEqual(
            {'outer_key0': {
                'inner_key0': None,
                'inner_key1': None,
                'inner_key2': None,
                'inner_key3': None},
             'outer_key1': {
                'inner_key0': None,
                'inner_key1': None,
                'inner_key2': None,
                'inner_key3': None},
             '': {
                'inner_key0': None,
                'inner_key1': None,
                'inner_key2': None,
                'inner_key3': None},
             'outer_key3': {
                'inner_key0': None,
                'inner_key1': 1,
                'inner_key2': 2,
                'inner_key3': 3}},
            place_utils._parse_rp_options(
                ['outer_key0', 'outer_key1::::', '::::', 'outer_key3::1:2:3'],
                ('inner_key0', 'inner_key1', 'inner_key2', 'inner_key3')),
        )

        self.assertRaises(
            ValueError,
            place_utils._parse_rp_options,
            ['outer_key0:'], ('inner_key1', 'inner_key2'),
        )

        self.assertRaises(
            ValueError,
            place_utils._parse_rp_options,
            ['outer_key0::'], ('inner_key1',),
        )

    def test_parse_rp_inventory_defaults(self):
        self.assertEqual(
            {},
            place_utils.parse_rp_inventory_defaults({}),
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_inventory_defaults,
            {'allocation_ratio': '-1.0'}
        )

        self.assertEqual(
            {'allocation_ratio': 1.0},
            place_utils.parse_rp_inventory_defaults(
                {'allocation_ratio': '1.0'}),
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_inventory_defaults,
            {'min_unit': '-1'}
        )

        self.assertEqual(
            {'min_unit': 1},
            place_utils.parse_rp_inventory_defaults(
                {'min_unit': '1'}),
        )

        self.assertRaises(
            ValueError,
            place_utils.parse_rp_inventory_defaults,
            {'no such inventory parameter': 1}
        )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters


ALIAS = 'security-groups-rules-belongs-to-default-sg'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = "Security group rule belongs to the project's default security group"
DESCRIPTION = ("Flag to determine if the security group rule belongs to the "
               "project's default security group")
UPDATED_TIMESTAMP = '2023-05-23T10:00:00-00:00'
BELONGS_TO_DEFAULT_SG = 'belongs_to_default_sg'

RESOURCE_ATTRIBUTE_MAP = {
    'security_group_rules': {
        BELONGS_TO_DEFAULT_SG: {
            'allow_post': False,
            'allow_put': False,
            'convert_to': converters.convert_to_boolean_if_not_none,
            'is_visible': True,
            'is_filter': True,
            'is_sort_key': False,
        },
    }
}

SUB_RESOURCE_ATTRIBUTE_MAP = {
}

ACTION_MAP = {
}

ACTION_STATUS = {
}

REQUIRED_EXTENSIONS = [
    'security-group'
]

OPTIONAL_EXTENSIONS = [
]



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from neutron_lib.api.definitions import port as port_def
from neutron_lib.api.definitions import uplink_status_propagation as usp


ALIAS = 'uplink-status-propagation-updatable'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Uplink status propagation updatable extension'
API_PREFIX = ''
DESCRIPTION = 'Provides ability to update the port uplink status.'
UPDATED_TIMESTAMP = '2024-09-03T18:00:00-00:00'
PROPAGATE_UPLINK_STATUS = usp.PROPAGATE_UPLINK_STATUS
RESOURCE_NAME = port_def.RESOURCE_NAME
COLLECTION_NAME = port_def.COLLECTION_NAME


propagate_uplink_status = copy.deepcopy(
    usp.RESOURCE_ATTRIBUTE_MAP[COLLECTION_NAME][PROPAGATE_UPLINK_STATUS])
propagate_uplink_status['allow_put'] = True

RESOURCE_ATTRIBUTE_MAP = {
    COLLECTION_NAME: {PROPAGATE_UPLINK_STATUS: propagate_uplink_status},
}

SUB_RESOURCE_ATTRIBUTE_MAP = {}
ACTION_MAP = {}
REQUIRED_EXTENSIONS = [usp.ALIAS]
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# Copyright (c) 2013 OpenStack Foundation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from webob import exc

from neutron_lib._i18n import _
from neutron_lib.api import converters
from neutron_lib.api.definitions import provider_net as pnet
from neutron_lib import constants


def convert_and_validate_segments(segments, valid_values=None):
    for segment in segments:
        segment.setdefault(pnet.NETWORK_TYPE, constants.ATTR_NOT_SPECIFIED)
        segment.setdefault(pnet.PHYSICAL_NETWORK, constants.ATTR_NOT_SPECIFIED)
        segmentation_id = segment.get(pnet.SEGMENTATION_ID)
        if segmentation_id:
            segment[pnet.SEGMENTATION_ID] = converters.convert_to_int(
                segmentation_id)
        else:
            segment[pnet.SEGMENTATION_ID] = constants.ATTR_NOT_SPECIFIED
        if len(segment) != 3:
            msg = (_("Unrecognized attribute(s) '%s'") %
                   ', '.join(set(segment.keys()) -
                             {pnet.NETWORK_TYPE, pnet.PHYSICAL_NETWORK,
                              pnet.SEGMENTATION_ID}))
            raise exc.HTTPBadRequest(msg)



# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib._i18n import _
from neutron_lib import exceptions


class AddressScopeNotFound(exceptions.NotFound):
    message = _("Address scope %(address_scope_id)s could not be found.")


class AddressScopeInUse(exceptions.InUse):
    message = _("Unable to complete operation on "
                "address scope %(address_scope_id)s. There are one or more "
                "subnet pools in use on the address scope.")


class AddressScopeUpdateError(exceptions.BadRequest):
    message = _("Unable to update address scope %(address_scope_id)s : "
                "%(reason)s.")


class NetworkAddressScopeAffinityError(exceptions.BadRequest):
    message = _("Subnets of the same address family hosted on the same "
                "network must all participate in the same address scope.")



# Copyright 2019 Ericsson Software Technology
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import extraroute
from neutron_lib.api.definitions import l3


ALIAS = 'extraroute-atomic'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Atomically add/remove extra routes'
DESCRIPTION = ('Edit extra routes of a router on server side by atomically '
               'adding/removing extra routes')
UPDATED_TIMESTAMP = '2019-07-10T00:00:00+00:00'
RESOURCE_ATTRIBUTE_MAP = {
    l3.ROUTERS: {}
}
SUB_RESOURCE_ATTRIBUTE_MAP = None
ACTION_MAP = l3.ACTION_MAP
ACTION_MAP[l3.ROUTER].update({
    'add_extraroutes': 'PUT',
    'remove_extraroutes': 'PUT',
})
REQUIRED_EXTENSIONS = [l3.ALIAS, extraroute.ALIAS]
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# Copyright 2023 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutron_lib.api import converters
from neutron_lib.api.definitions import l3
from neutron_lib.api.definitions import l3_ext_gw_multihoming
from neutron_lib import constants

ENABLE_DEFAULT_ROUTE_ECMP = 'enable_default_route_ecmp'

ALIAS = constants.L3_ENABLE_DEFAULT_ROUTE_ECMP_ALIAS
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Enable Default Route ECMP'
API_PREFIX = ''
DESCRIPTION = 'Enables configurable ECMP behavior for default routes'
UPDATED_TIMESTAMP = '2023-02-27T00:00:00-00:00'
RESOURCE_NAME = l3.ROUTER
COLLECTION_NAME = l3.ROUTERS
RESOURCE_ATTRIBUTE_MAP = {
    COLLECTION_NAME: {
        ENABLE_DEFAULT_ROUTE_ECMP: {
            'allow_post': True,
            'allow_put': True,
            'default': None,
            'is_visible': True,
            'enforce_policy': True,
            'convert_to': converters.convert_to_boolean_if_not_none,
        }
    }
}
SUB_RESOURCE_ATTRIBUTE_MAP = {}
ACTION_MAP = {}
REQUIRED_EXTENSIONS = [l3.ALIAS, l3_ext_gw_multihoming.ALIAS]
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# A declaration of a dummy neutron extension: this may have a number of
# constants being defined, and their aim is to document as much about
# the extension as possible.

# The alias of the extension.
ALIAS = 'dummy'

# Whether or not this extension is simply signaling behavior to the user
# or it actively modifies the attribute map (mandatory).
IS_SHIM_EXTENSION = False

# Whether the extension is marking the adoption of standardattr model for
# legacy resources, or introducing new standardattr attributes. False or
# None if the standardattr model is adopted since the introduction of
# resource extension (mandatory).
# If this is True, the alias for the extension should be prefixed with
# 'standard-attr-'.
IS_STANDARD_ATTR_EXTENSION = False

# The name of the extension (mandatory).
NAME = 'Foo Extension'

# A prefix for API resources. An empty prefix means that the API is going
# to be exposed at the v2/ level as any other core resource (mandatory).
API_PREFIX = ''

# The description of the extension (mandatory).
DESCRIPTION = "Provides support for foo"

# A timestamp of when the extension was introduced (mandatory).
UPDATED_TIMESTAMP = "2000-00-01T00:00:00-00:00"

# The specific resources and/or attributes for the extension (optional).
# In case of simple extensions, with single resource, the string constants
# RESOURCE_NAME and COLLECTION_NAME can be used, otherwise string literals
# can be used instead.

# The name of the resource introduced or being extended
# (in case it is defined by another extension, or it is
# a core resource).
RESOURCE_NAME = 'foo'

# The plural for the resource introduced or being extended
# (in case it is defined by another extension, or it is a
# core resource).
COLLECTION_NAME = 'fooes'

# The resource attribute map for the extension. It is effectively the
# bulk of the API contract alongside ACTION_MAP (mandatory).
RESOURCE_ATTRIBUTE_MAP = {
    COLLECTION_NAME: {
        'id': {'allow_post': False, 'allow_put': False,
               'validate': {'type:uuid': None},
               'is_visible': True, 'primary_key': True},
    }
}

# The subresource attribute map for the extension. It adds child resources
# to main extension's resource. The subresource map must have a parent and
# a parameters entry. If an extension does not need such a map, None can
# be specified (mandatory).
# Note that if an existing sub-resource is being extended, the
# existing resources to extend the new extension attributes must be
# defined under the 'parameters' key.
SUB_RESOURCE_ATTRIBUTE_MAP = {
    'subfoo': {
        'parent': {
            'collection_name': COLLECTION_NAME,
            'member_name': RESOURCE_NAME},
        'parameters': {
            'id': {'allow_post': False, 'allow_put': False,
                   'validate': {'type:uuid': None},
                   'is_visible': True,
                   'primary_key': True},
        },
    },
}

# The action map: it associates verbs with methods to be performed on
# the API resource (mandatory). For example:
#
# ACTION_MAP = {
#     RESOURCE_NAME: {
#        'add_my_foo_bars': 'PUT',
#        'remove_my_foo_bars': 'PUT',
#        'get_my_foo_bars': 'GET'
#    }
# }
ACTION_MAP = {
}

# The action status: it associates response statuses with methods to be
# performed on the API resource (mandatory). For example:
#
# ACTION_STATUS = {
#     'create': 201,
#     'delete': 204
# }
ACTION_STATUS = {
}

# The list of required extensions (mandatory).
REQUIRED_EXTENSIONS = [
]

# The list of optional extensions (mandatory).
OPTIONAL_EXTENSIONS = [
]

# TODO(armax): add support for modeling custom queries



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from oslo_config import cfg
import oslo_messaging as messaging
from oslo_messaging import conffixture as messaging_conffixture
from oslo_messaging import exceptions as oslomsg_exc
import testtools

from neutron_lib import fixture
from neutron_lib import rpc
from neutron_lib.tests import _base as base


CONF = cfg.CONF


class TestRPC(base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.useFixture(fixture.RPCFixture())

    @mock.patch.object(rpc, 'RequestContextSerializer')
    @mock.patch.object(messaging, 'get_rpc_transport')
    @mock.patch.object(messaging, 'get_notification_transport')
    def test_init(self, mock_noti_trans, mock_trans, mock_ser):
        transport = mock.Mock()
        noti_transport = mock.Mock()
        conf = mock.Mock()

        mock_trans.return_value = transport
        mock_noti_trans.return_value = noti_transport

        rpc.init(conf, rpc_ext_mods=['foo'])

        expected_mods = list(set(['foo'] + rpc._DFT_EXMODS))
        mock_trans.assert_called_once_with(
            conf, allowed_remote_exmods=expected_mods)
        mock_noti_trans.assert_called_once_with(
            conf, allowed_remote_exmods=expected_mods)
        self.assertIsNotNone(rpc.TRANSPORT)
        self.assertIsNotNone(rpc.NOTIFICATION_TRANSPORT)

    def test_cleanup_transport_null(self):
        rpc.NOTIFICATION_TRANSPORT = mock.Mock()
        rpc.TRANSPORT = None
        self.assertRaises(AssertionError, rpc.cleanup)
        rpc.TRANSPORT = mock.Mock()

    def test_cleanup_notification_transport_null(self):
        rpc.TRANSPORT = mock.Mock()
        rpc.NOTIFICATION_TRANSPORT = None
        self.assertRaises(AssertionError, rpc.cleanup)
        rpc.NOTIFICATION_TRANSPORT = mock.Mock()

    def test_cleanup(self):
        rpc.NOTIFICATION_TRANSPORT = mock.Mock()
        rpc.TRANSPORT = mock.Mock()
        trans_cleanup = mock.Mock()
        not_trans_cleanup = mock.Mock()
        rpc.TRANSPORT.cleanup = trans_cleanup
        rpc.NOTIFICATION_TRANSPORT.cleanup = not_trans_cleanup

        rpc.cleanup()

        trans_cleanup.assert_called_once_with()
        not_trans_cleanup.assert_called_once_with()
        self.assertIsNone(rpc.TRANSPORT)
        self.assertIsNone(rpc.NOTIFICATION_TRANSPORT)

        rpc.TRANSPORT = mock.Mock()
        rpc.NOTIFICATION_TRANSPORT = mock.Mock()

    @mock.patch.object(rpc, 'RequestContextSerializer')
    @mock.patch.object(messaging, 'get_rpc_client')
    def test_get_client(self, mock_get, mock_ser):
        rpc.TRANSPORT = mock.Mock()
        tgt = mock.Mock()
        ser = mock.Mock()
        mock_get.return_value = 'client'
        mock_ser.return_value = ser

        client = rpc.get_client(tgt, version_cap='1.0', serializer='foo')

        mock_ser.assert_called_once_with('foo')
        mock_get.assert_called_once_with(rpc.TRANSPORT,
                                         tgt, version_cap='1.0',
                                         serializer=ser,
                                         client_cls=rpc.BackingOffClient)
        self.assertEqual('client', client)

    @mock.patch.object(rpc, 'RequestContextSerializer')
    @mock.patch.object(messaging, 'get_rpc_server')
    def test_get_server(self, mock_get, mock_ser):
        ser = mock.Mock()
        tgt = mock.Mock()
        ends = mock.Mock()
        mock_ser.return_value = ser
        mock_get.return_value = 'server'

        server = rpc.get_server(tgt, ends, serializer='foo')

        mock_ser.assert_called_once_with('foo')
        mock_get.assert_called_once_with(rpc.TRANSPORT, tgt, ends,
                                         serializer=ser)
        self.assertEqual('server', server)

    def test_get_notifier(self):
        mock_notifier = mock.Mock(return_value=None)
        messaging.Notifier.__init__ = mock_notifier
        rpc.get_notifier('service', publisher_id='foo')
        mock_notifier.assert_called_once_with(
            mock.ANY, serializer=mock.ANY, publisher_id='foo')

    def test_get_notifier_null_publisher(self):
        mock_notifier = mock.Mock(return_value=None)
        messaging.Notifier.__init__ = mock_notifier
        rpc.get_notifier('service', host='bar')
        mock_notifier.assert_called_once_with(
            mock.ANY, serializer=mock.ANY, publisher_id='service.bar')


class TestRequestContextSerializer(base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.mock_base = mock.Mock()
        self.ser = rpc.RequestContextSerializer(self.mock_base)
        self.ser_null = rpc.RequestContextSerializer(None)

    def test_serialize_entity(self):
        self.mock_base.serialize_entity.return_value = 'foo'

        ser_ent = self.ser.serialize_entity('context', 'entity')

        self.mock_base.serialize_entity.assert_called_once_with('context',
                                                                'entity')
        self.assertEqual('foo', ser_ent)

    def test_deserialize_entity(self):
        self.mock_base.deserialize_entity.return_value = 'foo'

        deser_ent = self.ser.deserialize_entity('context', 'entity')

        self.mock_base.deserialize_entity.assert_called_once_with('context',
                                                                  'entity')
        self.assertEqual('foo', deser_ent)

    def test_deserialize_entity_null_base(self):
        deser_ent = self.ser_null.deserialize_entity('context', 'entity')

        self.assertEqual('entity', deser_ent)

    def test_serialize_context(self):
        context = mock.Mock()

        self.ser.serialize_context(context)

        context.to_dict.assert_called_once_with()

    def test_deserialize_context(self):
        context_dict = {'foo': 'bar',
                        'user_id': 1,
                        'tenant_id': 1,
                        'is_admin': True}

        c = self.ser.deserialize_context(context_dict)

        self.assertEqual(1, c.user_id)
        self.assertEqual(1, c.project_id)

    def test_deserialize_context_no_user_id(self):
        context_dict = {'foo': 'bar',
                        'user': 1,
                        'tenant_id': 1,
                        'is_admin': True}

        c = self.ser.deserialize_context(context_dict)

        self.assertEqual(1, c.user_id)
        self.assertEqual(1, c.project_id)

    def test_deserialize_context_no_tenant_id(self):
        context_dict = {'foo': 'bar',
                        'user_id': 1,
                        'project_id': 1,
                        'is_admin': True}

        c = self.ser.deserialize_context(context_dict)

        self.assertEqual(1, c.user_id)
        self.assertEqual(1, c.project_id)

    def test_deserialize_context_no_ids(self):
        context_dict = {'foo': 'bar', 'is_admin': True}

        c = self.ser.deserialize_context(context_dict)

        self.assertIsNone(c.user_id)
        self.assertIsNone(c.project_id)


class ServiceTestCase(base.BaseTestCase):
    # the class cannot be based on BaseTestCase since it mocks rpc.Connection

    def setUp(self):
        super().setUp()
        self.host = 'foo'
        self.topic = 'neutron-agent'

        self.target_mock = mock.patch('oslo_messaging.Target')
        self.target_mock.start()

        self.messaging_conf = messaging_conffixture.ConfFixture(CONF)
        self.messaging_conf.transport_url = 'fake://'
        self.messaging_conf.response_timeout = 0
        self.useFixture(self.messaging_conf)

        self.addCleanup(rpc.cleanup)
        rpc.init(CONF)

    @mock.patch.object(cfg, 'CONF')
    def test_operations(self, mock_conf):
        mock_conf.host = self.host
        with mock.patch('oslo_messaging.get_rpc_server') as get_rpc_server:
            rpc_server = get_rpc_server.return_value

            service = rpc.Service(self.host, self.topic)
            service.start()
            rpc_server.start.assert_called_once_with()

            service.stop()
            rpc_server.stop.assert_called_once_with()
            rpc_server.wait.assert_called_once_with()


class TimeoutTestCase(base.BaseTestCase):
    def setUp(self):
        super().setUp()

        self.messaging_conf = messaging_conffixture.ConfFixture(CONF)
        self.messaging_conf.transport_url = 'fake://'
        self.messaging_conf.response_timeout = 0
        self.useFixture(self.messaging_conf)

        self.addCleanup(rpc.cleanup)
        rpc.init(CONF)
        rpc.TRANSPORT = mock.MagicMock()
        rpc.TRANSPORT._send.side_effect = messaging.MessagingTimeout
        rpc.TRANSPORT.conf.oslo_messaging_metrics.metrics_enabled = False
        target = messaging.Target(version='1.0', topic='testing')
        self.client = rpc.get_client(target)
        self.call_context = mock.Mock()
        self.sleep = mock.patch('time.sleep').start()
        rpc.TRANSPORT.conf.rpc_response_timeout = 10
        rpc.TRANSPORT.conf.rpc_response_max_timeout = 300

    def test_timeout_unaffected_when_explicitly_set(self):
        rpc.TRANSPORT.conf.rpc_response_timeout = 5
        ctx = self.client.prepare(topic='sandwiches', timeout=77)
        with testtools.ExpectedException(messaging.MessagingTimeout):
            ctx.call(self.call_context, 'create_pb_and_j')
        # ensure that the timeout was not increased and the back-off sleep
        # wasn't called
        self.assertEqual(
            5,
            rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['create_pb_and_j'])
        self.assertFalse(self.sleep.called)

    def test_timeout_store_defaults(self):
        # any method should default to the configured timeout
        self.assertEqual(
            rpc.TRANSPORT.conf.rpc_response_timeout,
            rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_1'])
        self.assertEqual(
            rpc.TRANSPORT.conf.rpc_response_timeout,
            rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_2'])
        # a change to an existing should not affect new or existing ones
        rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_2'] = 7000
        self.assertEqual(
            rpc.TRANSPORT.conf.rpc_response_timeout,
            rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_1'])
        self.assertEqual(
            rpc.TRANSPORT.conf.rpc_response_timeout,
            rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_3'])

    def test_method_timeout_sleep(self):
        rpc.TRANSPORT.conf.rpc_response_timeout = 2
        for i in range(100):
            with testtools.ExpectedException(messaging.MessagingTimeout):
                self.client.call(self.call_context, 'method_1')
            # sleep value should always be between 0 and configured timeout
            self.assertGreaterEqual(self.sleep.call_args_list[0][0][0], 0)
            self.assertLessEqual(self.sleep.call_args_list[0][0][0], 2)
            self.sleep.reset_mock()

    def test_method_timeout_increases_on_timeout_exception(self):
        rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_1'] = 1
        for i in range(5):
            with testtools.ExpectedException(messaging.MessagingTimeout):
                self.client.call(self.call_context, 'method_1')

        # we only care to check the timeouts sent to the transport
        timeouts = [call[1]['timeout']
                    for call in rpc.TRANSPORT._send.call_args_list]
        self.assertEqual([1, 2, 4, 8, 16], timeouts)

    def test_method_timeout_config_ceiling(self):
        rpc.TRANSPORT.conf.rpc_response_timeout = 10
        # 5 doublings should max out at the 10xdefault ceiling
        for i in range(5):
            with testtools.ExpectedException(messaging.MessagingTimeout):
                self.client.call(self.call_context, 'method_1')
        self.assertEqual(
            rpc.TRANSPORT.conf.rpc_response_max_timeout,
            rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_1'])
        with testtools.ExpectedException(messaging.MessagingTimeout):
            self.client.call(self.call_context, 'method_1')
        self.assertEqual(
            rpc.TRANSPORT.conf.rpc_response_max_timeout,
            rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_1'])

    def test_timeout_unchanged_on_other_exception(self):
        rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_1'] = 1
        rpc.TRANSPORT._send.side_effect = ValueError
        with testtools.ExpectedException(ValueError):
            self.client.call(self.call_context, 'method_1')
        rpc.TRANSPORT._send.side_effect = messaging.MessagingTimeout
        with testtools.ExpectedException(messaging.MessagingTimeout):
            self.client.call(self.call_context, 'method_1')
        timeouts = [call[1]['timeout']
                    for call in rpc.TRANSPORT._send.call_args_list]
        self.assertEqual([1, 1], timeouts)

    def test_timeouts_for_methods_tracked_independently(self):
        rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_1'] = 1
        rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_2'] = 1
        for method in ('method_1', 'method_1', 'method_2',
                       'method_1', 'method_2'):
            with testtools.ExpectedException(messaging.MessagingTimeout):
                self.client.call(self.call_context, method)
        timeouts = [call[1]['timeout']
                    for call in rpc.TRANSPORT._send.call_args_list]
        self.assertEqual([1, 2, 1, 4, 2], timeouts)

    def test_timeouts_for_namespaces_tracked_independently(self):
        rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['ns1.method'] = 1
        rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['ns2.method'] = 1
        for ns in ('ns1', 'ns2'):
            self.client.target.namespace = ns
            for i in range(4):
                with testtools.ExpectedException(messaging.MessagingTimeout):
                    self.client.call(self.call_context, 'method')
        timeouts = [call[1]['timeout']
                    for call in rpc.TRANSPORT._send.call_args_list]
        self.assertEqual([1, 2, 4, 8, 1, 2, 4, 8], timeouts)

    def test_method_timeout_increases_with_prepare(self):
        rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_1'] = 1
        ctx = self.client.prepare(version='1.4')
        with testtools.ExpectedException(messaging.MessagingTimeout):
            ctx.call(self.call_context, 'method_1')
        with testtools.ExpectedException(messaging.MessagingTimeout):
            ctx.call(self.call_context, 'method_1')

        # we only care to check the timeouts sent to the transport
        timeouts = [call[1]['timeout']
                    for call in rpc.TRANSPORT._send.call_args_list]
        self.assertEqual([1, 2], timeouts)

    def test_set_max_timeout_caps_all_methods(self):
        rpc.TRANSPORT.conf.rpc_response_timeout = 300
        rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_1'] = 100
        rpc.BackingOffClient.set_max_timeout(50)
        # both explicitly tracked
        self.assertEqual(
            50, rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_1'])
        # as well as new methods
        self.assertEqual(
            50, rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_2'])

    def test_set_max_timeout_retains_lower_timeouts(self):
        rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_1'] = 10
        rpc.BackingOffClient.set_max_timeout(50)
        self.assertEqual(
            10, rpc._BackingOffContextWrapper._METHOD_TIMEOUTS['method_1'])

    def test_set_max_timeout_overrides_default_timeout(self):
        rpc.TRANSPORT.conf.rpc_response_timeout = 10
        self.assertEqual(
            rpc.TRANSPORT.conf.rpc_response_max_timeout,
            rpc._BackingOffContextWrapper.get_max_timeout())
        rpc._BackingOffContextWrapper.set_max_timeout(10)
        self.assertEqual(10, rpc._BackingOffContextWrapper.get_max_timeout())


class CastExceptionTestCase(base.BaseTestCase):
    def setUp(self):
        super().setUp()

        self.messaging_conf = messaging_conffixture.ConfFixture(CONF)
        self.messaging_conf.transport_url = 'fake://'
        self.messaging_conf.response_timeout = 0
        self.useFixture(self.messaging_conf)

        self.addCleanup(rpc.cleanup)
        rpc.init(CONF)
        rpc.TRANSPORT = mock.MagicMock()
        rpc.TRANSPORT._send.side_effect = oslomsg_exc.MessageDeliveryFailure
        rpc.TRANSPORT.conf.oslo_messaging_metrics.metrics_enabled = False
        target = messaging.Target(version='1.0', topic='testing')
        self.client = rpc.get_client(target)
        self.cast_context = mock.Mock()

    def test_cast_catches_exception(self):
        self.client.cast(self.cast_context, 'method_1')


class TestConnection(base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.conn = rpc.Connection()

    @mock.patch.object(messaging, 'Target')
    @mock.patch.object(cfg, 'CONF')
    @mock.patch.object(rpc, 'get_server')
    def test_create_consumer(self, mock_get, mock_cfg, mock_tgt):
        mock_cfg.host = 'foo'
        server = mock.Mock()
        target = mock.Mock()
        mock_get.return_value = server
        mock_tgt.return_value = target

        self.conn.create_consumer('topic', 'endpoints', fanout=True)

        mock_tgt.assert_called_once_with(topic='topic', server='foo',
                                         fanout=True)
        mock_get.assert_called_once_with(target, 'endpoints')
        self.assertEqual([server], self.conn.servers)

    def test_consume_in_threads(self):
        self.conn.servers = [mock.Mock(), mock.Mock()]

        servs = self.conn.consume_in_threads()

        for serv in self.conn.servers:
            serv.start.assert_called_once_with()
        self.assertEqual(servs, self.conn.servers)

    def test_close(self):
        self.conn.servers = [mock.Mock(), mock.Mock()]

        self.conn.close()

        for serv in self.conn.servers:
            serv.stop.assert_called_once_with()
            serv.wait.assert_called_once_with()



# Copyright (c) 2013 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import platform
import random
import time

from debtcollector import removals
import netaddr

from neutron_lib.utils import helpers
from neutron_lib.utils import net


class UnorderedList(list):
    """A list that is equals to any permutation of itself."""

    def __eq__(self, other):
        if not isinstance(other, list):
            return False
        return (sorted(self, key=helpers.safe_sort_key) ==
                sorted(other, key=helpers.safe_sort_key))

    def __neq__(self, other):
        return not self == other


@removals.remove
def is_bsd():
    """Return True on BSD-based systems."""

    system = platform.system()
    if system == 'Darwin':
        return True
    if 'bsd' in system.lower():
        return True
    return False


def get_random_cidr(version=4):
    if version == 4:
        return '10.%d.%d.0/%d' % (random.randint(3, 254),
                                  random.randint(3, 254),
                                  24)
    return '2001:db8:%x::/%d' % (random.getrandbits(16), 64)


def reset_random_seed():
    # reset random seed to make sure other processes extracting values from RNG
    # don't get the same results (useful especially when you then use the
    # random values to allocate system resources from global pool, like ports
    # to listen). Use both current time and pid to make sure no tests started
    # at the same time get the same values from RNG
    seed = time.time() + os.getpid()
    random.seed(seed)


def get_random_EUI():
    return netaddr.EUI(
        net.get_random_mac(['fe', '16', '3e', '00', '00', '00'])
    )


def get_random_ip_network(version=4):
    return netaddr.IPNetwork(get_random_cidr(version=version))



# Copyright 2013 VMware, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import l3


SERVICE_TYPE_ID = 'service_type_id'

ALIAS = 'router-service-type'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Router Service Type'
API_PREFIX = ''
DESCRIPTION = 'Provides router service type'
UPDATED_TIMESTAMP = '2013-01-29T00:00:00-00:00'
RESOURCE_ATTRIBUTE_MAP = {
    l3.ROUTERS: {
        SERVICE_TYPE_ID: {
            'allow_post': True,
            'allow_put': False,
            'validate': {
                'type:uuid_or_none': None
            },
            'default': None,
            'is_visible': True},
    }
}
SUB_RESOURCE_ATTRIBUTE_MAP = {}
ACTION_MAP = {}
REQUIRED_EXTENSIONS = [l3.ALIAS]
OPTIONAL_EXTENSIONS = []
ACTION_STATUS = {}



# Copyright (c) 2024 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from neutron_lib.api import converters
from neutron_lib.api.definitions import network
from neutron_lib import constants

ALIAS = 'qinq'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = "QinQ networks"
DESCRIPTION = (
    "Expose network 'qinq' attribute in the API. "
    "This allows to configure network to allow vlan in vlan "
    "configuration using 0x8a88 ethertype (QinQ)."
)
UPDATED_TIMESTAMP = "2024-12-02T10:00:00-00:00"
RESOURCE_NAME = network.RESOURCE_NAME
COLLECTION_NAME = network.COLLECTION_NAME
QINQ_FIELD = 'qinq'
RESOURCE_ATTRIBUTE_MAP = {
    COLLECTION_NAME: {
        QINQ_FIELD: {
            'allow_post': True,
            'allow_put': False,
            'convert_to': converters.convert_to_boolean,
            'default': constants.ATTR_NOT_SPECIFIED,
            'is_visible': True,
            'is_filter': True,
            'validate': {'type:boolean': None}
            }
    },
}
SUB_RESOURCE_ATTRIBUTE_MAP = None
ACTION_MAP = {}
ACTION_STATUS = {}
REQUIRED_EXTENSIONS = []
OPTIONAL_EXTENSIONS = []



# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib._i18n import _
from neutron_lib import exceptions


class PlacementEndpointNotFound(exceptions.NotFound):
    message = _("Placement API endpoint not found.")


class PlacementResourceNotFound(exceptions.NotFound):
    message = _("Placement resource not found on url: %(url)s.")


class PlacementResourceProviderNotFound(exceptions.NotFound):
    message = _("Placement resource provider not found %(resource_provider)s.")


class PlacementResourceProviderGenerationConflict(exceptions.Conflict):
    message = _("Placement resource provider generation does not match with "
                "the server side for resource provider: "
                "%(resource_provider)s with generation %(generation)s.")


class PlacementInventoryNotFound(exceptions.NotFound):
    message = _("Placement inventory not found for resource provider "
                "%(resource_provider)s, resource class %(resource_class)s.")


class PlacementInventoryUpdateConflict(exceptions.Conflict):
    message = _("Placement inventory update conflict for resource provider "
                "%(resource_provider)s, resource class %(resource_class)s.")


class PlacementAggregateNotFound(exceptions.NotFound):
    message = _("Aggregate not found for resource provider "
                "%(resource_provider)s.")


class PlacementTraitNotFound(exceptions.NotFound):
    message = _("Placement trait not found %(trait)s.")


class PlacementResourceClassNotFound(exceptions.NotFound):
    message = _("Placement resource class not found %(resource_class)s")


class PlacementAPIVersionIncorrect(exceptions.NotFound):
    message = _("Placement API version %(current_version)s, do not meet the "
                "needed version %(needed_version)s.")


class PlacementResourceProviderNameNotUnique(exceptions.Conflict):
    message = _("Another resource provider exists with the provided name: "
                "%(name)s.")


class PlacementClientError(exceptions.NeutronException):
    message = _("Placement Client Error (4xx): %(msg)s")


class UnknownResourceProvider(exceptions.BadRequest):
    """Resource provider not known by neutron backends."""
    message = _("No such resource provider known by Neutron: %(rsc_provider)s")


class AmbiguousResponsibilityForResourceProvider(exceptions.NeutronException):
    """Not clear who's responsible for resource provider."""
    message = _("Expected one driver to be responsible for resource provider "
                "%(rsc_provider)s, but got many: %(drivers)s")


class PlacementAllocationGenerationConflict(exceptions.Conflict):
    message = _("Resource allocation has been changed for consumer "
                "%(consumer)s in Placement while Neutron tried to update it.")


class PlacementAllocationRemoved(exceptions.BadRequest):
    message = _("Resource allocation is deleted for consumer %(consumer)s")


class PlacementAllocationRpNotExists(exceptions.BadRequest):
    message = _("Resource provider %(resource_provider)s for %(consumer)s "
                "does not exist")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc


class WorkerBase:

    @property
    def _workers(self):
        try:
            return self.__workers
        except AttributeError:
            self.__workers = []
        return self.__workers

    def get_workers(self):
        """Returns a collection NeutronWorker instances needed by this service.

        """
        return list(self._workers)

    def add_worker(self, worker):
        """Adds NeutronWorker needed for this service

        If a object needs to define workers thread/processes outside of API/RPC
        workers then it will call this method to register worker. Should be
        called on initialization stage before running services
        """
        self._workers.append(worker)

    def add_workers(self, workers):
        """Adds NeutronWorker list needed for this service

        The same as add_worker but adds a list of workers
        """
        self._workers.extend(workers)


class ServicePluginBase(WorkerBase, metaclass=abc.ABCMeta):
    """Define base interface for any Advanced Service plugin."""
    supported_extension_aliases = []

    @classmethod
    def __subclasshook__(cls, klass):
        """Checking plugin class.

        The __subclasshook__ method is a class method
        that will be called every time a class is tested
        using issubclass(klass, ServicePluginBase).
        In that case, it will check that every method
        marked with the abstractmethod decorator is
        provided by the plugin class.
        """

        if not cls.__abstractmethods__:
            return NotImplemented

        for method in cls.__abstractmethods__:
            if any(method in base.__dict__ for base in klass.__mro__):
                continue
            return NotImplemented
        return True

    @abc.abstractmethod
    def get_plugin_type(self):
        """Return one of predefined service types.

        """
        pass

    @abc.abstractmethod
    def get_plugin_description(self):
        """Return string description of the plugin."""
        pass



# Copyright (c) 2018 Intel Corporation.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters
from neutron_lib.api.definitions import provider_net as providernet
from neutron_lib import constants
from neutron_lib.db import constants as db_const

# The name of the extension.
NAME = 'Neutron Network Segment Range'
# The alias of the extension.
ALIAS = 'network-segment-range'
# The description of the extension.
DESCRIPTION = "Provides support for the network segment range management"

# A timestamp of when the extension was introduced.
UPDATED_TIMESTAMP = "2018-11-29T00:00:00-00:00"

# The name of the resource.
RESOURCE_NAME = 'network_segment_range'
# The plural for the resource.
COLLECTION_NAME = 'network_segment_ranges'

# Min ID for VLAN, VXLAN, GRE and GENEVE all equal to 1; Max ID for them are
# 4094, 2 ** 24 - 1, 2 ** 32 - 1 and 2 ** 24 - 1 respectively.
# Take the largest range: [MIN_GRE_ID, MAX_GRE_ID] as the limit for validation.
NETWORK_SEGMENT_RANGE_LIMIT = [constants.MIN_GRE_ID, constants.MAX_GRE_ID]

RESOURCE_ATTRIBUTE_MAP = {
    COLLECTION_NAME: {
        'id': {'allow_post': False,
               'allow_put': False,
               'validate': {'type:uuid': None},
               'is_filter': True,
               'is_sort_key': True,
               'primary_key': True,
               'is_visible': True},
        'name': {'allow_post': True,
                 'allow_put': True,
                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                 'default': '',
                 'is_filter': True,
                 'is_sort_key': True,
                 'is_visible': True},
        'default': {'allow_post': False,
                    'allow_put': False,
                    'convert_to': converters.convert_to_boolean,
                    'default': False,
                    'is_visible': True},
        constants.SHARED: {'allow_post': True,
                           'allow_put': False,
                           'convert_to': converters.convert_to_boolean,
                           'default': True,
                           'is_visible': True},
        'project_id': {'allow_post': True,
                       'allow_put': False,
                       'validate': {
                           'type:string': db_const.PROJECT_ID_FIELD_SIZE},
                       'required_by_policy': True,
                       'is_filter': True,
                       'is_sort_key': True,
                       'is_visible': True},
        'network_type': {'allow_post': True,
                         'allow_put': False,
                         'validate': {
                             'type:values':
                                 constants.NETWORK_SEGMENT_RANGE_TYPES},
                         'default': constants.ATTR_NOT_SPECIFIED,
                         'is_filter': True,
                         'is_visible': True},
        'physical_network': {'allow_post': True,
                             'allow_put': False,
                             'validate': {
                                 'type:string':
                                     providernet.PHYSICAL_NETWORK_MAX_LEN},
                             'default': constants.ATTR_NOT_SPECIFIED,
                             'is_filter': True,
                             'is_visible': True},
        'minimum': {'allow_post': True,
                    'allow_put': True,
                    'convert_to': converters.convert_to_int,
                    'validate': {'type:range': NETWORK_SEGMENT_RANGE_LIMIT},
                    'is_visible': True},
        'maximum': {'allow_post': True,
                    'allow_put': True,
                    'convert_to': converters.convert_to_int,
                    'validate': {'type:range': NETWORK_SEGMENT_RANGE_LIMIT},
                    'is_visible': True},
        'used': {'allow_post': False,
                 'allow_put': False,
                 'is_visible': True},
        'available': {'allow_post': False,
                      'allow_put': False,
                      'convert_to': converters.convert_none_to_empty_list,
                      'is_visible': True}
    }
}

# Whether or not this extension is simply signaling behavior to the user
# or it actively modifies the attribute map.
IS_SHIM_EXTENSION = False

# Whether the extension is marking the adoption of standardattr model for
# legacy resources, or introducing new standardattr attributes. False or
# None if the standardattr model is adopted since the introduction of
# resource extension.
# If this is True, the alias for the extension should be prefixed with
# 'standard-attr-'.
IS_STANDARD_ATTR_EXTENSION = False

# The subresource attribute map for the extension. It adds child resources
# to main extension's resource. The subresource map must have a parent and
# a parameters entry. If an extension does not need such a map, None can
# be specified (mandatory).
SUB_RESOURCE_ATTRIBUTE_MAP = {}

# The action map: it associates verbs with methods to be performed on
# the API resource.
ACTION_MAP = {}

# The list of required extensions.
REQUIRED_EXTENSIONS = [providernet.ALIAS]

# The list of optional extensions.
OPTIONAL_EXTENSIONS = []

# The action status.
ACTION_STATUS = {}



# Copyright (c) 2014 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import re

from hacking import core


_all_log_levels = {'critical', 'error', 'exception', 'info',
                   'warning', 'debug'}
_all_hints = {'_LC', '_LE', '_LI', '_', '_LW'}


_log_warn = re.compile(
    r"(.)*LOG\.(warn)\(\s*('|\")")


_log_translation_hint = re.compile(
    r".*LOG\.({levels})\(\s*({hints})\(".format(
        levels='|'.join(_all_log_levels),
        hints='|'.join(_all_hints),
    ))


def _translation_checks_not_enforced(filename):
    # Do not do these validations on tests
    return any(pat in filename for pat in ["/tests/", "rally-jobs/plugins/"])


@core.flake8ext
def check_log_warn_deprecated(logical_line, filename):
    """N532 - Use LOG.warning due to compatibility with py3.

    :param logical_line: The logical line to check.
    :param filename: The file name where the logical line exists.
    :returns: None if the logical line passes the check, otherwise a tuple
        is yielded that contains the offending index in logical line and a
        message describe the check validation failure.
    """
    msg = "N532: Use LOG.warning due to compatibility with py3"
    if _log_warn.match(logical_line):
        yield (0, msg)


@core.flake8ext
def check_raised_localized_exceptions(logical_line, filename):
    """N534 - Untranslated exception message.

    :param logical_line: The logical line to check.
    :param filename: The file name where the logical line exists.
    :returns: None if the logical line passes the check, otherwise a tuple
        is yielded that contains the offending index in logical line and a
        message describe the check validation failure.
    """
    if _translation_checks_not_enforced(filename):
        return

    logical_line = logical_line.strip()
    raised_search = re.compile(
        r"raise (?:\w*)\((.*)\)").match(logical_line)
    if raised_search:
        exception_msg = raised_search.groups()[0]
        if exception_msg.startswith("\"") or exception_msg.startswith("\'"):
            msg = "N534: Untranslated exception message."
            yield (logical_line.index(exception_msg), msg)


@core.flake8ext
def no_translate_logs(logical_line, filename):
    """N537 - Don't translate logs.

    Check for 'LOG.*(_(' and 'LOG.*(_Lx('

    Translators don't provide translations for log messages, and operators
    asked not to translate them.

    * This check assumes that 'LOG' is a logger.

    :param logical_line: The logical line to check.
    :param filename: The file name where the logical line exists.
    :returns: None if the logical line passes the check, otherwise a tuple
        is yielded that contains the offending index in logical line and a
        message describe the check validation failure.
    """
    if _translation_checks_not_enforced(filename):
        return

    msg = "N537: Log messages should not be translated!"
    match = _log_translation_hint.match(logical_line)
    if match:
        yield (logical_line.index(match.group()), msg)



# Copyright 2011 VMware, Inc, 2015 A10 Networks, Inc
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Neutron base exception handling.
"""

from oslo_utils import excutils

from neutron_lib._i18n import _


class NeutronException(Exception):
    """Base Neutron Exception.

    To correctly use this class, inherit from it and define
    a 'message' property. That message will get printf'd
    with the keyword arguments provided to the constructor.
    """
    message = _("An unknown exception occurred.")

    def __init__(self, **kwargs):
        try:
            super().__init__(self.message % kwargs)
            self.msg = self.message % kwargs
        except Exception:
            with excutils.save_and_reraise_exception() as ctxt:
                if not self.use_fatal_exceptions():
                    ctxt.reraise = False
                    # at least get the core message out if something happened
                    super().__init__(self.message)

    def __str__(self):
        return self.msg

    def use_fatal_exceptions(self):
        """Is the instance using fatal exceptions.

        :returns: Always returns False.
        """
        return False


class BadRequest(NeutronException):
    """An exception indicating a generic bad request for a said resource.

    A generic exception indicating a bad request for a specified resource.

    :param resource: The resource requested.
    :param msg: A message indicating why the request is bad.
    """
    message = _('Bad %(resource)s request: %(msg)s.')


class NotFound(NeutronException):
    """A generic not found exception."""
    pass


class Conflict(NeutronException):
    """A generic conflict exception."""
    pass


class NotAuthorized(NeutronException):
    """A generic not authorized exception."""
    message = _("Not authorized.")


class ServiceUnavailable(NeutronException):
    """A generic service unavailable exception."""
    message = _("The service is unavailable.")


class AdminRequired(NotAuthorized):
    """A not authorized exception indicating an admin is required.

    A specialization of the NotAuthorized exception that indicates and admin
    is required to carry out the operation or access a resource.

    :param reason: A message indicating additional details on why admin is
        required for the operation access.
    """
    message = _("User does not have admin privileges: %(reason)s.")


class ObjectNotFound(NotFound):
    """A not found exception indicating an identifiable object isn't found.

    A specialization of the NotFound exception indicating an object with a said
    ID doesn't exist.

    :param id: The ID of the (not found) object.
    """
    message = _("Object %(id)s not found.")


class NetworkNotFound(NotFound):
    """An exception indicating a network was not found.

    A specialization of the NotFound exception indicating a requested network
    could not be found.

    :param net_id: The UUID of the (not found) network requested.
    """
    message = _("Network %(net_id)s could not be found.")


class SubnetNotFound(NotFound):
    """An exception for a requested subnet that's not found.

    A specialization of the NotFound exception indicating a requested subnet
    could not be found.

    :param subnet_id: The UUID of the (not found) subnet that was requested.
    """
    message = _("Subnet %(subnet_id)s could not be found.")


class PortNotFound(NotFound):
    """An exception for a requested port that's not found.

    A specialization of the NotFound exception indicating a requested port
    could not be found.

    :param port_id: The UUID of the (not found) port that was requested.
    """
    message = _("Port %(port_id)s could not be found.")


class PortNotFoundOnNetwork(NotFound):
    """An exception for a requested port on a network that's not found.

    A specialization of the NotFound exception that indicates a specified
    port on a specified network doesn't exist.

    :param port_id: The UUID of the (not found) port that was requested.
    :param net_id: The UUID of the network that was requested for the port.
    """
    message = _("Port %(port_id)s could not be found "
                "on network %(net_id)s.")


class DeviceNotFoundError(NotFound):
    """An exception for a requested device that's not found.

    A specialization of the NotFound exception indicating a requested device
    could not be found.

    :param device_name: The name of the (not found) device that was requested.
    """
    message = _("Device '%(device_name)s' does not exist.")


class InUse(NeutronException):
    """A generic exception indicating a resource is already in use."""
    message = _("The resource is in use.")


class NetworkInUse(InUse):
    """An operational error indicating the network still has ports in use.

    A specialization of the InUse exception indicating a network operation was
    requested, but failed because there are still ports in use on the said
    network.

    :param net_id: The UUID of the network requested.
    :param reason: Details on why the operation failed. If None, a default
        reason is used indicating one or more ports still in use
        on the network.
    """
    message = _("Unable to complete operation on network %(net_id)s. "
                "%(reason)s.")

    def __init__(self, **kwargs):
        if 'reason' not in kwargs:
            kwargs['reason'] = _("There are one or more ports still in use "
                                 "on the network")
        super().__init__(**kwargs)


class SubnetInUse(InUse):
    """An operational error indicating a subnet is still in use.

    A specialization of the InUse exception indicating an operation failed
    on a subnet because the subnet is still in use.

    :param subnet_id: The UUID of the subnet requested.
    :param reason: Details on why the operation failed. If None, a default
        reason is used indicating one or more ports still have IP allocations
        on the subnet.
    """
    message = _("Unable to complete operation on subnet %(subnet_id)s: "
                "%(reason)s.")

    def __init__(self, **kwargs):
        if 'reason' not in kwargs:
            kwargs['reason'] = _("One or more ports have an IP allocation "
                                 "from this subnet")
        super().__init__(**kwargs)


class SubnetPoolInUse(InUse):
    """An operational error indicating a subnet pool is still in use.

    A specialization of the InUse exception indicating an operation failed
    on a subnet pool because it's still in use.

    :param subnet_pool_id: The UUID of the subnet pool requested.
    :param reason: Details on why the operation failed. If None a default
        reason is used indicating two or more concurrent subnets are allocated.
    """
    message = _("Unable to complete operation on subnet pool "
                "%(subnet_pool_id)s. %(reason)s.")

    def __init__(self, **kwargs):
        if 'reason' not in kwargs:
            kwargs['reason'] = _("Two or more concurrent subnets allocated")
        super().__init__(**kwargs)


class PortInUse(InUse):
    """An operational error indicating a requested port is already attached.

    A specialization of the InUse exception indicating an operation failed on
    a port because it already has an attached device.

    :param port_id: The UUID of the port requested.
    :param net_id: The UUID of the requested port's network.
    :param device_id: The UUID of the device already attached to the port.
    """
    message = _("Unable to complete operation on port %(port_id)s "
                "for network %(net_id)s. Port already has an attached "
                "device %(device_id)s.")


class ServicePortInUse(InUse):
    """An error indicating a service port can't be deleted.

    A specialization of the InUse exception indicating a requested service
    port can't be deleted via the APIs.

    :param port_id: The UUID of the port requested.
    :param reason: Details on why the operation failed.
    """
    message = _("Port %(port_id)s cannot be deleted directly via the "
                "port API: %(reason)s.")


class PortBound(InUse):
    """An operational error indicating a port is already bound.

    A specialization of the InUse exception indicating an operation can't
    complete because the port is already bound.

    :param port_id: The UUID of the port requested.
    :param vif_type: The VIF type associated with the bound port.
    :param old_mac: The old MAC address of the port.
    :param net_mac: The new MAC address of the port.
    """
    message = _("Unable to complete operation on port %(port_id)s, "
                "port is already bound, port type: %(vif_type)s, "
                "old_mac %(old_mac)s, new_mac %(new_mac)s.")


class PortBoundNUMAAffinityPolicy(InUse):
    """An operational error indicating a port is already bound.

    NUMA affinity policy cannot be modified when the port is bound.

    :param port_id: The UUID of the port requested.
    :param host_id: The host ID where the port is bound.
    :param numa_affinity_policy: value passed to be updated.
    """
    message = _("Unable to complete operation on port %(port_id)s, "
                "port is already bound to host %(host_id)s, "
                "numa_affinity_policy value given %(numa_affinity_policy)s.")


class MacAddressInUse(InUse):
    """An network operational error indicating a MAC address is already in use.

    A specialization of the InUse exception indicating an operation failed
    on a network because a specified MAC address is already in use on that
    network.

    :param net_id: The UUID of the network.
    :param mac: The requested MAC address that's already in use.
    """
    message = _("Unable to complete operation for network %(net_id)s. "
                "The mac address %(mac)s is in use.")


class InvalidIpForNetwork(BadRequest):
    """An exception indicating an invalid IP was specified for a network.

    A specialization of the BadRequest exception indicating a specified IP
    address is invalid for a network.

    :param ip_address: The IP address that's invalid on the network.
    """
    message = _("IP address %(ip_address)s is not a valid IP "
                "for any of the subnets on the specified network.")


class InvalidIpForSubnet(BadRequest):
    """An exception indicating an invalid IP was specified for a subnet.

    A specialization of the BadRequest exception indicating a specified IP
    address is invalid for a subnet.

    :param ip_address: The IP address that's invalid on the subnet.
    """
    message = _("IP address %(ip_address)s is not a valid IP "
                "for the specified subnet.")


class IpAddressInUse(InUse):
    """An network operational error indicating an IP address is already in use.

    A specialization of the InUse exception indicating an operation can't
    complete because an IP address is in use.

    :param net_id: The UUID of the network.
    :param ip_address: The IP address that's already in use on the network.
    """
    message = _("Unable to complete operation for network %(net_id)s. "
                "The IP address %(ip_address)s is in use.")


class VlanIdInUse(InUse):
    """A network operational error indicating a VLAN ID is already in use.

    A specialization of the InUse exception indicating network creation failed
    because a specified VLAN is already in use on the physical network.

    :param vlan_id: The VLAN ID.
    :param physical_network: The physical network.
    """
    message = _("Unable to create the network. "
                "The VLAN %(vlan_id)s on physical network "
                "%(physical_network)s is in use.")


class TunnelIdInUse(InUse):
    """A network creation failure due to tunnel ID already in use.

    A specialization of the InUse exception indicating network creation failed
    because a said tunnel ID is already in use.

    :param tunnel_id: The ID of the tunnel that's already in use.
    """
    message = _("Unable to create the network. "
                "The tunnel ID %(tunnel_id)s is in use.")


class ResourceExhausted(ServiceUnavailable):
    """A service unavailable error indicating a resource is exhausted."""
    pass


class NoNetworkAvailable(ResourceExhausted):
    """A failure to create a network due to no tenant networks for allocation.

    A specialization of the ResourceExhausted exception indicating network
    creation failed because no tenant network are available for allocation.
    """
    message = _("Unable to create the network. "
                "No tenant network is available for allocation.")


class SubnetMismatchForPort(BadRequest):
    """A bad request error indicating a specified subnet isn't on a port.

    A specialization of the BadRequest exception indicating a subnet on a port
    doesn't match a specified subnet.

    :param port_id: The UUID of the port.
    :param subnet_id: The UUID of the requested subnet.
    """
    message = _("Subnet on port %(port_id)s does not match "
                "the requested subnet %(subnet_id)s.")


class Invalid(NeutronException):
    """A generic base class for invalid errors."""
    def __init__(self, message=None):
        self.message = message
        super().__init__()


class InvalidInput(BadRequest):
    """A bad request due to invalid input.

    A specialization of the BadRequest error indicating bad input was
    specified.

    :param error_message: Details on the operation that failed due to bad
        input.
    """
    message = _("Invalid input for operation: %(error_message)s.")


class IpAddressGenerationFailure(Conflict):
    """A conflict error due to no more IP addresses on a said network.

    :param net_id: The UUID of the network that has no more IP addresses.
    """
    message = _("No more IP addresses available on network %(net_id)s.")


class PreexistingDeviceFailure(NeutronException):
    """A creation error due to an already existing device.

    An exception indication creation failed due to an already existing
    device.

    :param dev_name: The device name that already exists.
    """
    message = _("Creation failed. %(dev_name)s already exists.")


class OverQuota(Conflict):
    """A error due to exceeding quota limits.

    A specialization of the Conflict exception indicating quota has been
    exceeded.

    :param overs: The resources that have exceeded quota.
    """
    message = _("Quota exceeded for resources: %(overs)s.")


class InvalidContentType(NeutronException):
    """An error due to invalid content type.

    :param content_type: The invalid content type.
    """
    message = _("Invalid content type %(content_type)s.")


class ExternalIpAddressExhausted(BadRequest):
    """An error due to not finding IP addresses on an external network.

    A specialization of the BadRequest exception indicating no IP addresses
    can be found on a network.

    :param net_id: The UUID of the network.
    """
    message = _("Unable to find any IP address on external "
                "network %(net_id)s.")


class InvalidConfigurationOption(NeutronException):
    """An error due to an invalid configuration option value.

    :param opt_name: The name of the configuration option that has an invalid
        value.
    :param opt_value: The value that's invalid for the configuration option.
    """
    message = _("An invalid value was provided for %(opt_name)s: "
                "%(opt_value)s.")


class NetworkTunnelRangeError(NeutronException):
    """An error due to an invalid network tunnel range.

    An exception indicating an invalid network tunnel range was specified.

    :param tunnel_range: The invalid tunnel range. If specified in the
        start:end' format, they will be converted automatically.
    :param error: Additional details on why the range is invalid.
    """
    message = _("Invalid network tunnel range: "
                "'%(tunnel_range)s' - %(error)s.")

    def __init__(self, **kwargs):
        # Convert tunnel_range tuple to 'start:end' format for display
        if isinstance(kwargs['tunnel_range'], tuple):
            kwargs['tunnel_range'] = "%d:%d" % kwargs['tunnel_range']
        super().__init__(**kwargs)


class PolicyInitError(NeutronException):
    """An error due to policy initialization failure.

    :param policy: The policy that failed to initialize.
    :param reason: Details on why the policy failed to initialize.
    """
    message = _("Failed to initialize policy %(policy)s because %(reason)s.")


class PolicyCheckError(NeutronException):
    """An error due to a policy check failure.

    :param policy: The policy that failed to check.
    :param reason: Additional details on the failure.
    """
    message = _("Failed to check policy %(policy)s because %(reason)s.")


class MultipleExceptions(Exception):
    """Container for multiple exceptions encountered.

    The API layer of Neutron will automatically unpack, translate,
    filter, and combine the inner exceptions in any exception derived
    from this class.
    """

    def __init__(self, exceptions, *args, **kwargs):
        """Create a new instance wrapping the exceptions.

        :param exceptions: The inner exceptions this instance is composed of.
        :param args: Passed onto parent constructor.
        :param kwargs: Passed onto parent constructor.
        """
        super().__init__(*args, **kwargs)
        self.inner_exceptions = exceptions


class HostMacAddressGenerationFailure(ServiceUnavailable):
    """MAC address generation failure for a host.

    :param host: The host MAC address generation failed for.
    """
    message = _("Unable to generate unique mac for host %(host)s.")


class NetworkMacAddressGenerationFailure(ServiceUnavailable):
    """An error related to MAC address generation on a network.

        :param net_id: The ID of the network MAC address generation failed on.
        """
    message = _("Unable to generate unique mac on network %(net_id)s.")


class InvalidServiceType(InvalidInput):
    """An error due to an invalid service type.

    :param service_type: The service type that's invalid.
    """
    message = _("Invalid service type: %(service_type)s.")


class NetworkVlanRangeError(NeutronException):
    message = _("Invalid network VLAN range: '%(vlan_range)s' - '%(error)s'.")

    def __init__(self, **kwargs):
        # Convert vlan_range tuple to 'start:end' format for display
        if isinstance(kwargs['vlan_range'], tuple):
            kwargs['vlan_range'] = "%d:%d" % kwargs['vlan_range']
        super().__init__(**kwargs)


class PhysicalNetworkNameError(NeutronException):
    message = _("Empty physical network name.")


class TenantIdProjectIdFilterConflict(BadRequest):
    message = _("Both tenant_id and project_id passed as filters.")


class SubnetPoolNotFound(NotFound):
    message = _("Subnet pool %(subnetpool_id)s could not be found.")


class StateInvalid(BadRequest):
    message = _("Unsupported port state: %(port_state)s.")


class DhcpPortInUse(InUse):
    message = _("Port %(port_id)s is already acquired by another DHCP agent")


class HostRoutesExhausted(BadRequest):
    # NOTE(xchenum): probably make sense to use quota exceeded exception?
    message = _("Unable to complete operation for %(subnet_id)s. "
                "The number of host routes exceeds the limit %(quota)s.")


class DNSNameServersExhausted(BadRequest):
    # NOTE(xchenum): probably make sense to use quota exceeded exception?
    message = _("Unable to complete operation for %(subnet_id)s. "
                "The number of DNS nameservers exceeds the limit %(quota)s.")


class FlatNetworkInUse(InUse):
    message = _("Unable to create the flat network. "
                "Physical network %(physical_network)s is in use.")


class NoNetworkFoundInMaximumAllowedAttempts(ServiceUnavailable):
    message = _("Unable to create the network. "
                "No available network found in maximum allowed attempts.")


class MalformedRequestBody(BadRequest):
    message = _("Malformed request body: %(reason)s.")


class InvalidAllocationPool(BadRequest):
    message = _("The allocation pool %(pool)s is not valid.")


class UnsupportedPortDeviceOwner(Conflict):
    message = _("Operation %(op)s is not supported for device_owner "
                "%(device_owner)s on port %(port_id)s.")


class OverlappingAllocationPools(Conflict):
    message = _("Found overlapping allocation pools: "
                "%(pool_1)s %(pool_2)s for subnet %(subnet_cidr)s.")


class OutOfBoundsAllocationPool(BadRequest):
    message = _("The allocation pool %(pool)s spans "
                "beyond the subnet cidr %(subnet_cidr)s.")


class BridgeDoesNotExist(NeutronException):
    message = _("Bridge %(bridge)s does not exist.")


class QuotaResourceUnknown(NotFound):
    message = _("Unknown quota resources %(unknown)s.")


class QuotaMissingTenant(BadRequest):
    message = _("Tenant-id was missing from quota request.")


class InvalidQuotaValue(Conflict):
    message = _("Change would make usage less than 0 for the following "
                "resources: %(unders)s.")


class InvalidSharedSetting(Conflict):
    message = _("Unable to reconfigure sharing settings for network "
                "%(network)s. Multiple tenants are using it.")


class ExtensionsNotFound(NotFound):
    message = _("Extensions not found: %(extensions)s.")


class GatewayConflictWithAllocationPools(InUse):
    message = _("Gateway ip %(ip_address)s conflicts with "
                "allocation pool %(pool)s.")


class GatewayIpInUse(InUse):
    message = _("Current gateway ip %(ip_address)s already in use "
                "by port %(port_id)s. Unable to update.")


class NetworkVxlanPortRangeError(NeutronException):
    message = _("Invalid network VXLAN port range: '%(vxlan_range)s'.")


class VxlanNetworkUnsupported(NeutronException):
    message = _("VXLAN network unsupported.")


class DuplicatedExtension(NeutronException):
    message = _("Found duplicate extension: %(alias)s.")


class DriverCallError(MultipleExceptions):
    def __init__(self, exc_list=None):
        super().__init__(exc_list or [])


class DeviceIDNotOwnedByTenant(Conflict):
    message = _("The following device_id %(device_id)s is not owned by your "
                "tenant or matches another tenants router.")


class InvalidCIDR(BadRequest):
    message = _("Invalid CIDR %(input)s given as IP prefix.")


class FailToDropPrivilegesExit(SystemExit):
    """Exit exception raised when a drop privileges action fails."""
    code = 99


class NetworkIdOrRouterIdRequiredError(NeutronException):
    message = _('Both network_id and router_id are None. '
                'One must be provided.')


class EmptySubnetPoolPrefixList(BadRequest):
    message = _("Empty subnet pool prefix list.")


class PrefixVersionMismatch(BadRequest):
    message = _("Cannot mix IPv4 and IPv6 prefixes in a subnet pool.")


class UnsupportedMinSubnetPoolPrefix(BadRequest):
    message = _("Prefix '%(prefix)s' not supported in IPv%(version)s pool.")


class IllegalSubnetPoolPrefixBounds(BadRequest):
    message = _("Illegal prefix bounds: %(prefix_type)s=%(prefixlen)s, "
                "%(base_prefix_type)s=%(base_prefixlen)s.")


class IllegalSubnetPoolPrefixUpdate(BadRequest):
    message = _("Illegal update to prefixes: %(msg)s.")


class SubnetAllocationError(NeutronException):
    message = _("Failed to allocate subnet: %(reason)s.")


class AddressScopePrefixConflict(Conflict):
    message = _("Failed to associate address scope: subnetpools "
                "within an address scope must have unique prefixes.")


class IllegalSubnetPoolAssociationToAddressScope(BadRequest):
    message = _("Illegal subnetpool association: subnetpool %(subnetpool_id)s "
                "cannot be associated with address scope "
                "%(address_scope_id)s.")


class IllegalSubnetPoolIpVersionAssociationToAddressScope(BadRequest):
    message = _("Illegal subnetpool association: subnetpool %(subnetpool_id)s "
                "cannot associate with address scope %(address_scope_id)s "
                "because subnetpool ip_version is not %(ip_version)s.")


class IllegalSubnetPoolUpdate(BadRequest):
    message = _("Illegal subnetpool update : %(reason)s.")


class MinPrefixSubnetAllocationError(BadRequest):
    message = _("Unable to allocate subnet with prefix length %(prefixlen)s, "
                "minimum allowed prefix is %(min_prefixlen)s.")


class MaxPrefixSubnetAllocationError(BadRequest):
    message = _("Unable to allocate subnet with prefix length %(prefixlen)s, "
                "maximum allowed prefix is %(max_prefixlen)s.")


class SubnetPoolDeleteError(BadRequest):
    message = _("Unable to delete subnet pool: %(reason)s.")


class SubnetPoolQuotaExceeded(OverQuota):
    message = _("Per-tenant subnet pool prefix quota exceeded.")


class NetworkSubnetPoolAffinityError(BadRequest):
    message = _("Subnets hosted on the same network must be allocated from "
                "the same subnet pool.")


class ObjectActionError(NeutronException):
    message = _('Object action %(action)s failed because: %(reason)s.')


class CTZoneExhaustedError(NeutronException):
    message = _("IPtables conntrack zones exhausted, iptables rules cannot "
                "be applied.")


class TenantQuotaNotFound(NotFound):
    message = _("Quota for project %(tenant_id)s could not be found.")


class MultipleFilterIDForIPFound(Conflict):
    message = _("Multiple filter IDs for IP %(ip)s found.")


class FilterIDForIPNotFound(NotFound):
    message = _("Filter ID for IP %(ip)s could not be found.")


class FailedToAddQdiscToDevice(NeutronException):
    message = _("Failed to add %(direction)s qdisc "
                "to device %(device)s.")


class PortBindingNotFound(NotFound):
    message = _("Binding for port %(port_id)s for host %(host)s could not be "
                "found.")


class PortBindingAlreadyActive(Conflict):
    message = _("Binding for port %(port_id)s on host %(host)s is already "
                "active.")


class PortBindingAlreadyExists(Conflict):
    message = _("Binding for port %(port_id)s on host %(host)s already "
                "exists.")


class PortBindingError(NeutronException):
    message = _("Binding for port %(port_id)s on host %(host)s could not be "
                "created or updated.")


class ProcessExecutionError(RuntimeError):
    def __init__(self, message, returncode):
        super().__init__(message)
        self.returncode = returncode


class InvalidSubnetServiceType(InvalidInput):
    message = _("Subnet service type %(service_type)s does not correspond "
                "to a valid device owner.")


class InvalidInputSubnetServiceType(InvalidInput):
    message = _("Subnet service type %(service_type)s is not a string.")



# Copyright 2015 Futurewei. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters
from neutron_lib import constants
from neutron_lib.db import constants as db_const

# The alias of the extension.
ALIAS = 'flow_classifier'

# Whether or not this extension is simply signaling behavior to the user
# or it actively modifies the attribute map.
IS_SHIM_EXTENSION = False

# Whether the extension is marking the adoption of standardattr model for
# legacy resources, or introducing new standardattr attributes. False or
# None if the standardattr model is adopted since the introduction of
# resource extension.
# If this is True, the alias for the extension should be prefixed with
# 'standard-attr-'.
IS_STANDARD_ATTR_EXTENSION = False

# The name of the extension.
NAME = 'Flow Classifier'

# A prefix for API resources. An empty prefix means that the API is going
# to be exposed at the v2/ level as any other core resource.
API_PREFIX = '/sfc'

# The description of the extension.
DESCRIPTION = ("API for selection of packets to direct through service "
               "function chains")

# A timestamp of when the extension was last updated.
UPDATED_TIMESTAMP = "2015-10-05T10:00:00-00:00"


FC_SUPPORTED_ETHERTYPES = [constants.IPv4, constants.IPv6]


RESOURCE_NAME = 'flow_classifier'
COLLECTION_NAME = 'flow_classifiers'


# Attribute Map
RESOURCE_ATTRIBUTE_MAP = {
    COLLECTION_NAME: {
        'id': {
            'allow_post': False, 'allow_put': False,
            'is_visible': True,
            'validate': {'type:uuid': None},
            'primary_key': True},
        'name': {
            'allow_post': True, 'allow_put': True,
            'is_visible': True, 'default': None,
            'validate': {'type:string': db_const.NAME_FIELD_SIZE},
            'convert_to': converters.convert_none_to_empty_string},
        'description': {
            'allow_post': True, 'allow_put': True,
            'is_visible': True, 'default': None,
            'validate': {'type:string': db_const.DESCRIPTION_FIELD_SIZE},
            'convert_to': converters.convert_none_to_empty_string},
        'tenant_id': {
            'allow_post': True, 'allow_put': False,
            'is_visible': True,
            'validate': {'type:string': db_const.PROJECT_ID_FIELD_SIZE},
            'required_by_policy': True},
        'ethertype': {
            'allow_post': True, 'allow_put': False,
            'is_visible': True, 'default': "IPv4",
            'default_overrides_none': True,
            'validate': {'type:values': FC_SUPPORTED_ETHERTYPES},
            'convert_to': converters.convert_uppercase_ip},
        'protocol': {
            'allow_post': True, 'allow_put': False,
            'is_visible': True, 'default': None,
            'convert_to': converters.convert_to_protocol},
        'source_port_range_min': {
            'allow_post': True, 'allow_put': False,
            'is_visible': True, 'default': None,
            'convert_to': converters.convert_to_int_if_not_none,
            'validate': {'type:range_or_none': [0, constants.PORT_MAX]}},
        'source_port_range_max': {
            'allow_post': True, 'allow_put': False,
            'is_visible': True, 'default': None,
            'convert_to': converters.convert_to_int_if_not_none,
            'validate': {'type:range_or_none': [0, constants.PORT_MAX]}},
        'destination_port_range_min': {
            'allow_post': True, 'allow_put': False,
            'is_visible': True, 'default': None,
            'convert_to': converters.convert_to_int_if_not_none,
            'validate': {'type:range_or_none': [0, constants.PORT_MAX]}},
        'destination_port_range_max': {
            'allow_post': True, 'allow_put': False,
            'is_visible': True, 'default': None,
            'convert_to': converters.convert_to_int_if_not_none,
            'validate': {'type:range_or_none': [0, constants.PORT_MAX]}},
        'source_ip_prefix': {
            'allow_post': True, 'allow_put': False,
            'is_visible': True, 'default': None,
            'validate': {'type:subnet_or_none': None}},
        'destination_ip_prefix': {
            'allow_post': True, 'allow_put': False,
            'is_visible': True, 'default': None,
            'validate': {'type:subnet_or_none': None}},
        'logical_source_port': {
            'allow_post': True, 'allow_put': False,
            'is_visible': True, 'default': None,
            'validate': {'type:uuid_or_none': None}},
        'logical_destination_port': {
            'allow_post': True, 'allow_put': False,
            'is_visible': True, 'default': None,
            'validate': {'type:uuid_or_none': None}},
        'l7_parameters': {
            # NOTE(tmorin): at this point, there is no clean way to extend
            # this attribute; it seems only used by GBP (by monkeypatching it
            # at runtime, see
            # http://codesearch.openstack.org/?q=AIM_FLC_L7_PARAMS)
            # So let's support the GBP fields by now, but consider that
            # future additions will need to happen via proper API extensions
            'allow_post': True, 'allow_put': False,
            'is_visible': True,
            'default': {}, 'default_overrides_none': True,
            'convert_to': converters.convert_none_to_empty_dict,
            'validate': {
                'type:dict': {
                    'logical_source_network': {
                        'default': None,
                        'type:uuid_or_none': None
                    },
                    'logical_destination_network': {
                        'default': None,
                        'type:uuid_or_none': None
                    }
                }
            }
        }
    },
}

# The subresource attribute map for the extension. It adds child resources
# to main extension's resource. The subresource map must have a parent and
# a parameters entry. If an extension does not need such a map, None can
# be specified (mandatory).
SUB_RESOURCE_ATTRIBUTE_MAP = None

# The action map: it associates verbs with methods to be performed on
# the API resource.
ACTION_MAP = {
}

# The action status.
ACTION_STATUS = {
}

# The list of required extensions.
REQUIRED_EXTENSIONS = [
]

# The list of optional extensions.
OPTIONAL_EXTENSIONS = [
]



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import fip_pf_description as pfw_desc
from neutron_lib.api.definitions import floating_ip_port_forwarding as pfw

ALIAS = 'floating-ip-port-forwarding-port-ranges'
IS_SHIM_EXTENSION = False
IS_STANDARD_ATTR_EXTENSION = False
NAME = 'Floating IP Port Forwarding support port ranges'
DESCRIPTION = 'Add support to port ranges'
UPDATED_TIMESTAMP = '2020-07-01T10:00:00-00:00'
EXTERNAL_PORT_RANGE = 'external_port_range'
INTERNAL_PORT_RANGE = 'internal_port_range'
RESOURCE_ATTRIBUTE_MAP = {}
SUB_RESOURCE_ATTRIBUTE_MAP = {
    pfw.COLLECTION_NAME: {
        'parameters': {
            EXTERNAL_PORT_RANGE: {
                'allow_post': True, 'allow_put': True,
                'validate': {'type:port_range': [1, 65535]},
                'is_visible': True,
                'is_sort_key': True,
                'is_filter': True,
                'default': None},
            INTERNAL_PORT_RANGE: {
                'allow_post': True, 'allow_put': True,
                'validate': {'type:port_range': [1, 65535]},
                'is_visible': True,
                'default': None},
            pfw.EXTERNAL_PORT: {
                'allow_post': True, 'allow_put': True,
                'validate': {'type:range_or_none': [1, 65535]},
                'is_visible': True,
                'is_sort_key': True,
                'is_filter': True,
                'default': None},
            pfw.INTERNAL_PORT: {
                'allow_post': True, 'allow_put': True,
                'validate': {'type:range_or_none': [1, 65535]},
                'is_visible': True,
                'default': None},
        }
    }
}
ACTION_MAP = {}
REQUIRED_EXTENSIONS = [pfw.ALIAS]
OPTIONAL_EXTENSIONS = [pfw_desc.ALIAS]
ACTION_STATUS = {}
